/*
 * SimpleInputMethodRequests.java - Input Method Requests for Simple Input Method Plug-in
 * :tabSize=8:indentSize=8:noTabs=false:
 * :folding=explicit:collapseFolds=1:
 *
 * Copyright (C) 2005 Yusuke SHIROYAMA
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.awt.*;
import java.awt.event.*;
import java.awt.im.*;
import java.awt.font.*;
import java.text.*;
import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.textarea.JEditTextArea;
import org.gjt.sp.jedit.textarea.TextAreaExtension;
import org.gjt.sp.jedit.textarea.TextAreaPainter;

public class SimpleInputMethodRequests extends TextAreaExtension implements InputMethodRequests
{
	EditPane  editPane;

	StringBuffer  ComposedText;
	AttributedString  ComposedAttrText, EmptyAttrText;
	Point   returnValue;

	public SimpleInputMethodRequests(EditPane editpane) {
		this.editPane = editpane;
		JEditTextArea  textArea = editPane.getTextArea();
		textArea.getPainter().addExtension(TextAreaPainter.HIGHEST_LAYER,this);

		textArea.enableInputMethods(true);
		ComposedText=new StringBuffer();
		ComposedAttrText = EmptyAttrText= new AttributedString("");

		textArea.addInputMethodListener(new InputMethodListener(){
			public void inputMethodTextChanged( InputMethodEvent event ){
				Buffer  buffer = editPane.getBuffer();
				JEditTextArea  textarea = editPane.getTextArea();
				AttributedCharacterIterator text = event.getText();
				int count = event.getCommittedCharacterCount();
				ComposedText.setLength(0);
				ComposedAttrText=EmptyAttrText;
				if( text!=null ){
					int i;
					char c;
					StringBuffer FixedText = new StringBuffer();
					for( i=0 ,c=text.first() ; i<count ; i++,c=text.next() ){
						FixedText.append(c);
						buffer.insert( textarea.getCaretPosition() , FixedText.toString() );
						FixedText.setLength(0);
					}

					for( i=0 ; c!=AttributedCharacterIterator.DONE ; c=text.next() ){
						ComposedText.append(c);
					}
					ComposedAttrText = new AttributedString( text , count , text.getEndIndex());
				}
				editPane.getTextArea().getPainter().repaint();
			}

			public void caretPositionChanged( InputMethodEvent event ){
			}
		});

		returnValue = new Point();
	}

	public Rectangle getTextLocation( TextHitInfo offset ){
		Rectangle r = new Rectangle(100,100 );
		Point p = editPane.getTextArea().getLocationOnScreen();
		r.translate(p.x,p.y);
		return r;
	}

	public TextHitInfo getLocationOffset( int x, int y ){
		return null;
	}

	public int getInsertPositionOffset(){
		return 0;
	}

	public AttributedCharacterIterator getCommittedText( int beginIndex , int endIndex , AttributedCharacterIterator.Attribute[] attributes ){
		return null;
	}

	public int getCommittedTextLength(){
		return 0;
	}

	public AttributedCharacterIterator cancelLatestCommittedText( AttributedCharacterIterator.Attribute[] attributes ){
		return null;
	}

	public AttributedCharacterIterator getSelectedText( AttributedCharacterIterator.Attribute[] attributes ){
		return null;
	}

	public void paintValidLine(Graphics2D gfx, int screenLine,
								int physicalLine, int start, int end, int y) {
		JEditTextArea    textArea = editPane.getTextArea();
		TextAreaPainter  painter = textArea.getPainter();
		int caret = textArea.getCaretPosition();
		if(caret < start || caret >= end)
			return;

		Font font = painter.getFont();
		FontMetrics  fm = painter.getFontMetrics();

		int offset = caret - textArea.getLineStartOffset(physicalLine);
		textArea.offsetToXY(physicalLine,offset,returnValue);
		int caretX = returnValue.x;
		int height = fm.getHeight();
		
		if( ComposedText.length()>0){
			ComposedAttrText.addAttribute(TextAttribute.FONT,font );
			gfx.setFont(font);
			int rect = gfx.getClipBounds().width;
			int stringwidth = fm.stringWidth(ComposedText.toString());
			if( stringwidth + caretX > rect ){
				gfx.setColor(painter.getBackground().darker());
				gfx.fillRect(rect-stringwidth,y,stringwidth,fm.getHeight());
				gfx.setColor(painter.getForeground());
				gfx.drawString(ComposedAttrText.getIterator(),rect-stringwidth,y+height-2);
			}else{
				gfx.setColor(painter.getBackground());
				gfx.fillRect(caretX+1,y,stringwidth,fm.getHeight());
				gfx.setColor(painter.getForeground());
				gfx.drawString(ComposedAttrText.getIterator(),caretX+1,y+height-2);

			}
		}
	}
}

